/**
 * Resources:
 * https://stackoverflow.com/questions/76131891/what-is-the-best-method-for-socket-io-authentication
 *
 */
import {Application, Request, Response} from 'express';
import UserService from '../services/UserService';
import jwt from "jsonwebtoken";
import config from "./Config";
import {loginAccountSchema, registerAccountSchema} from "./ZodTypes";
import path from "path";

function isValidAsset(assetName: string) {
    const assetPath = path.join(__dirname, 'public', 'assets', assetName);
    return assetPath.startsWith(path.join(__dirname, 'public', 'assets'));
}

async function addAuthRoutes(app: Application) {
    app.get('/assets/:asset', (req: Request, res: Response) => {
        const assetName = req.params.asset;

        if (!isValidAsset(assetName)) {
            return res.status(400).send('Invalid asset name');
        }

        const options = {
            root: path.join(__dirname, 'public', 'assets'),
        };

        res.sendFile(assetName, options, (err) => {
            if (err) {
                console.error('Error sending file:', err);
                res.status(500).send('Error downloading the asset');
            }
        });
    });

    app.post('/login', async (req: Request, res: Response) => {
        const { username, password } = req.body;

        try {
            loginAccountSchema.parse({ username, password });
        } catch (error: any) {
            return res.status(400).json({ message: error.errors[0].message });
        }

        const userService = new UserService();
        const user = await userService.login(username, password);

        if (user) { //test
            const token = jwt.sign({ id: user.id }, config.JWT_SECRET, { expiresIn: '1h' });
            return res.status(200).json({ token });
        }

        return res.status(400).json({ message: 'Failed to login' });
    });

    app.post('/register', async (req: Request, res: Response) => {
        const { username, password } = req.body;

        try {
            registerAccountSchema.parse({ username, password });
        } catch (error: any) {
            return res.status(400).json({ message: error.errors[0].message });
        }

        const userService = new UserService();
        const user = await userService.register(username, password);

        if (user) {
            const token = jwt.sign({ id: user.id }, config.JWT_SECRET, { expiresIn: '1h' });
            return res.status(200).json({ token });
        }

        return res.status(400).json({ message: 'Failed to register user' });
    });

    console.log('[✅] Auth routes added');
}

export { addAuthRoutes };